# cosmic_ray.cl -- Cosmic  ray  events in each input image are detected and replaced by
#                  the average of the four neighbors.
# Created:  Yoshifusa Ita, 02-May-07
# Modified: Takehiko Wada 04-Nov-07 replace un-phisical "large value"
#           with very small value (so that co-add ignore it...)
# Modified: Takehiko Wada 24-Nov-07 apply NIR too...
# Modified: Takehiko Wada 24-Nov-07 tmp.pl$B$,$N$3$C$F$$$k$H!"(BNIR$B$,$3$1$k(B??
# Modified: Takehiko Wada 24-Nov-07 tmp.pl$B$OD>A0$K:o=|$7$h$&!#(B1:$B%?%9%/(B# $B$N0BA4(B	$B!"(B2 debug
# Modified: Takehiko Wada 24-Nov-07 lacos$B$r$D$+$&%*%W%7%g%s$r$h$&$$$7$?$$(B
# Modiffed: Yoshifusa Ita 28-Nov-07 NIR$B$b(Bcosmic_ray$B$r$H$k$h$&$K$7$F$3!<$I$r$9$C$-$j2=(B
#                                   lacos$B$r;H$&>l9gMQ$K(Bheader$B$+$i(Bread noise$B$H(Bgain$B$H$C$F$/$k$h$&$K$7$?!#(B
# slit mask region$B$r(Bmask out... for lacos

procedure cosmic_ray (ircconst, logfile, prefixs, deltemp, verbose)
string  mode="al"

string ircconst {"constants.database", prompt="IRC constants database file name"}
string logfile  {"irclog", prompt="output of the mkirclog"}
string prefixs  {"a", prompt="prefix string"}
bool   deltemp = yes {prompt="Delete intermediate files?"}
bool   verbose = no {prompt="Print verbose progress messages?"}
string algoreject {"cosmicrays", prompt="cosmicrays|lacos"}

struct *list0

begin

 string fname, outname, libpath, irc_const, prefix
 bool verb, deltmp
 string text, detector, area, slit_area
 int npix
 real mean, median, modee, sigma
 real thresh, fluxratio
 string algo_reject	
 int expid
 real epadu, epadu_n, epadu_s, epadu_l
 real rnoise, rnoise_n, rnoise_s, rnoise_l


 print ("\n")
 print ("### COSMICRAY ###\n")


# get query parameters
 libpath = osfn ("irclib$")
 irc_const = ircconst
 prefix = prefixs
 verb = verbose
 deltmp = deltemp
 algo_reject = algoreject


# Read irc constants
 irc_const = libpath//irc_const

	
# make input file list
 print ("Making the input file list...\n")
 if( access("filename.list") ){ delete("filename.list", ver-) }
 ! cat irclog | perl -e 'while(<>){ chomp; @buf=split(/\s+/,$_); if($buf[0] !~ /\#/){ printf("%s\n",$buf[0])} }' > filename.list


# Subtracting scattering light pattern
 print ("Detect and replace cosmic rays...\n")
 print ("\t #This may take a while...\n")
 list0 = "filename.list"
 while( fscan(list0, fname) !=EOF){
	fname = prefix//fname//".fits"
	outname = "C"//fname
		
	if(verb){
		printf("debug: %s\n",fname)
	}
	if( access(outname) ){ imdel (outname, yes, verify=no, default_acti=yes) }

	# $B%X%C%@$rFI$s$G!"(BDETECTOR$B$rD4$Y$k(B
	hselect(fname, fields="DETECTOR", expr="yes") | scan(detector)
	hselect(fname, fields="EXPID", expr="yes") | scan(expid)

	# $B%$%a!<%8%(%j%"$NDj5A(B
	if(detector == "NIR"){
	        area = "[100:400,100:400]"
			slit_area = "[1:412,397:512]"
		hselect(fname, fields="GAIN"    , expr="yes") | scan(epadu_n)
		hselect(fname, fields="RNOISE"  , expr="yes") | scan(rnoise_n)
		epadu = epadu_n
		rnoise = rnoise_n
	}else if(detector == "MIRS"){
	        area = "[40:256,1:256]"
			slit_area ="[1:20,1:256]"
		hselect(fname, fields="GAIN"    , expr="yes") | scan(epadu_s)
		hselect(fname, fields="RNOISE"  , expr="yes") | scan(rnoise_s)
		epadu = epadu_s
		rnoise = rnoise_s
	}else if(detector == "MIRL"){
	        area = "[1:220,82:200]"
			slit_area ="[246:256,82:200]"

		hselect(fname, fields="GAIN"    , expr="yes") | scan(epadu_l)
		hselect(fname, fields="RNOISE"  , expr="yes") | scan(rnoise_l)
		epadu = epadu_l
		rnoise = rnoise_l
	}else{
		print("Detector name is wrong!!\n")
		bye
	}
	# imaging$BNN0h$N(Bstat$B$r$9$k(B
	imstat(fname//area, fields="npix,mean,midpt,mode,stddev", lower=0, upper=10000, lsigma=3, usigma=3, nclip=100, binwidth=0.1, format=no, cache=no) | scan(npix, mean, median, modee, sigma)
	# $B8!=P4oKh$K%Q%i%a!<%?$r$+$($k!)(B
	thresh = 3.0*sigma

	if(verb){
		printf("debug: %d %g %g %g %g\n",npix,mean,median,modee,sigma)
	}

	if(detector == "NIR"){
#		fluxratio = 3.0
		fluxratio = 10.0
#		fluxratio = 15.0
#		fluxratio = 20.0
	}else if(detector == "MIRS"){
		fluxratio = 3.0
	}else if(detector == "MIRL"){
		fluxratio = 3.0
	}else{
		print("Detector name is wrong!!\n")
		bye
	}

	# cosmicrays$B$r=|5n(B
	if( access("tmp.pl") ){ delete("tmp.pl", ver-) }
	if(algo_reject == "lacos"){
		stsdas						
		lacos_im(fname, outname, "tmp.pl", gain=epadu, readn=rnoise, statsec=area, skyval=0., sigclip=4.5, sigfrac=0.5, objlim=5.0, niter=4, verbose=verb) 

		imreplace(images=outname//slit_area, value=0,lower=INDEF, upper=INDEF)
#		imarith (outname//slit_area, "*", 0, outname, title="", divzero=0., hparams="", pixtype="real", calctype="double", verbose=verb, noact=no)
			
	}else{
		cosmicrays (fname, outname, "yes", crmasks="tmp.pl", threshold=thresh, fluxratio=fluxratio, npasses=5, window="5", interactive=no, train=no, objects="", savefile="", plotfile="", graphics="stdgraph", cursor="")

	}

	# dark.cl$B$G(Bmask $B$7$?(Bhot pixel$B$,;D$C$F$$$?$i!"!"!"(B
	# bad_const(0$B$r2>Dj(B)$B$G(Breplace           65536
#	imreplace(images=outname, value=0,lower=200000, upper=INDEF)

	# $BCf4V%U%!%$%k$r>C$7$?$$$H$-$O>C$9(B
        if (deltmp) imdel(fname, yes, verify=no, default_acti=yes)
 }

# cleaning
 if( access("tmp.pl") ){ delete("tmp.pl", ver-) }
 if( access("filename.list") ){ delete("filename.list", ver-) }

 print ("### COSMICRAY finished!! ###\n")

end
